unit Clipper;
{General clipper-like functions}

interface

uses
  DBTables, DbiProcs, DbiTypes;

function Trim(cString: string): string;
function AllTrim(cString: string): string;
function PadR(cString: string; nLen: integer): string;
function Replicate(c: char; nLen: integer): string;
function Space(nLen: integer): string;
function Empty(cString: string): boolean;
function LeftStr(cString: string; nLen: integer): string;
function Right(cString: string; nLen: integer): string;
function RecNo(table: TTable): longint;

implementation

function Trim(cString: string): string;
{trim off any spaces and return the string}
begin
  Result := cString;
  while (Ord(Result[0]) > 0) and (Result[Ord(Result[0])] = ' ') do
    Result[0] := Chr(Ord(Result[0]) - 1);
end;


function AllTrim(cString: string): string;
{trim off any spaces and return the string}
var
  startPos, endPos: integer;
begin
  startPos := 1;
  endPos   := Length(cString);
  while (startPos < Length(cString)) and (cString[startPos] = ' ') do
    Inc(startPos);
  while (endPos > startPos) and (cString[endPos] = ' ') do
    Dec(endPos);
  if (endPos = startPos) and (cString[endPos] = ' ') then
    Result := ''
  else
    Result := Copy(cString, startPos, endPos-startPos+1);
end;


function PadR(cString: string; nLen: integer): string;
{make certain the string is exactly nLen character}
begin
  if (Length(cString) <> nLen) then
    if Length(cString) > nLen then
      Result := Copy(cString,1,nLen)
    else
      Result := cString + Space(nLen-Length(cString))
  else
    Result := cString;
end;

function LeftStr(cString: string; nLen: integer): string;
{return the nLen left most substring}
begin
  if nLen = 0 then
    Result := ''
  else if (Length(cString) > nLen) then
    Result := Copy(cString,1,nLen)
  else
    Result := cString;
end;

function Right(cString: string; nLen: integer): string;
begin
  if nLen = 0 then
    Result := ''
  else if (Length(cString) > nLen) then
    Result := Copy(cString,Length(cString)-nLen+1, nLen)
  else
    Result := cString;
end;

function Space(nLen: integer): string;
{return the number of spaces asked for}
begin
  Result := Replicate(' ', nLen);
end;

function Replicate(c: char; nLen: integer): string;
{return a string containing nLen times the character c}
var
  i: integer;
begin
  Result := '';
  for i := 1 to nLen do
    Result := Result + c;
end;

function Empty(cString: string): boolean;
{returns true if the string is empty of non-spaces, false otherwise}
begin
  if Trim(cString) = '' then
    Result := true
  else
    Result := false;
end;

function RecNo(table: TTable): longint;
var
  myrecprop: RECprops;
begin
  table.updatecursorpos;
  dbiGetRecord(table.handle, dbiNOLOCK, nil, @myrecprop);
  Result := myrecprop.iPhyRecNum;
end;

end.

