unit DbTest1;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Menus, DB, DBTables, Buttons, ExtCtrls, DBCtrls, Grids,
  DBGrids, StdCtrls, DbiProcs, DbiTypes, Dialogs, Clipper;

type
  TDBFTest1 = class(TForm)
    MainMenu1: TMainMenu;
    Manager1: TMenuItem;
    Reindex1: TMenuItem;
    DataSource1: TDataSource;
    Panel1: TPanel;
    SpeedButton1: TSpeedButton;
    SpeedButton2: TSpeedButton;
    SpeedButton3: TSpeedButton;
    SpeedButton4: TSpeedButton;
    SpeedButton5: TSpeedButton;
    DBGrid1: TDBGrid;
    DBNavigator1: TDBNavigator;
    Label1: TLabel;
    DBFstatus: TLabel;
    Table1: TTable;
    Table1CRT_DATE: TDateField;
    Table1CRT_CODE: TStringField;
    Table1INFO1: TStringField;
    Table1INFO2: TStringField;
    Table1INFO3: TStringField;
    Table1INFO4: TStringField;
    AvailableIndexes: TComboBox;
    Label2: TLabel;
    Files: TMenuItem;
    OpenTable1: TMenuItem;
    CloseTable1: TMenuItem;
    N1: TMenuItem;
    SearchTable1: TMenuItem;
    N2: TMenuItem;
    Exit1: TMenuItem;
    DeleteMDX1: TMenuItem;
    SpeedButton6: TSpeedButton;
    Bevel1: TBevel;
    Label3: TLabel;
    RecordNo: TLabel;
    Label4: TLabel;
    Query1: TQuery;
    SpeedButton7: TSpeedButton;
    SpeedButton8: TSpeedButton;
    DataSource2: TDataSource;
    SpeedButton9: TSpeedButton;
    SpeedButton11: TSpeedButton;
    procedure OpenDBF(Sender: TObject);
    procedure CloseDBF(Sender: TObject);
    procedure ExitApp(Sender: TObject);
    procedure ReindexTest(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure AvailableIndexesChange(Sender: TObject);
    procedure DeleteMDX(Sender: TObject);
    procedure SearchTable1Click(Sender: TObject);
    procedure DBNavigator1Click(Sender: TObject; Button: TNavigateBtn);
    procedure DataSource1DataChange(Sender: TObject; Field: TField);
    procedure SpeedButton7Click(Sender: TObject);
    procedure SpeedButton8Click(Sender: TObject);
    procedure SpeedButton9Click(Sender: TObject);
    procedure SpeedButton11Click(Sender: TObject);
  private
    { Private declarations }
    procedure SetComboBox;
    procedure ShowRecord;
  public
    { Public declarations }
  end;

var
  DBFTest1: TDBFTest1;

const
  MAX_FIELDS = 15;

type
  TRecordStructure = record
    fldName: string;     {name of the field}
    fldType: TFieldType; {type of info for this field}
    fldPos:  integer;    {offset in Record buffer of field}
    fldLen:  integer;    {number of bytes field uses}
  end;

var
  tableInfo: array[1..MAX_FIELDS] of TRecordStructure;
  tableInfoCount: integer;

function FilterData(ulClientData : LongInt;
                    pRecBuf      : Pointer;
                    iPhyRecNum   : Longint
                    ): Integer; export;

implementation

{$R *.DFM}

uses
  Reindex, GetDate;

procedure TDBFTest1.OpenDBF(Sender: TObject);
begin
  if not Table1.Active then
    begin
      Table1.Open;
      DBFstatus.Caption := 'Open';
    end;
end;

procedure TDBFTest1.CloseDBF(Sender: TObject);
begin
  if Table1.Active then
  begin
    Table1.Close;
    DBFstatus.Caption := 'Close';
  end;
end;

procedure TDBFTest1.ExitApp(Sender: TObject);
begin
  CloseDBF(Sender);
  Close;
end;

procedure TDBFTest1.ReindexTest(Sender: TObject);
begin
  DBFstatus.Caption := 'Reindexing';
  CloseDBF(Sender);
  IndexFiles;
  OpenDBF(Sender);
  SetComboBox;
end;

procedure TDBFTest1.FormCreate(Sender: TObject);
begin
  LoadIndexes;
  SetComboBox;
end;

procedure TDBFTest1.SetComboBox;
var
  IndexList: TStringList;
begin
  {load TComboBox with all EXISTING tags from TTable, if any}
  IndexList := TStringList.Create;
  Table1.GetIndexNames(IndexList);
  AvailableIndexes.Items.Clear;
  AvailableIndexes.Items.AddStrings(IndexList);
  IndexList.Free;
end;

procedure TDBFTest1.FormDestroy(Sender: TObject);
begin
  FreeIndexes;
end;

procedure TDBFTest1.AvailableIndexesChange(Sender: TObject);
begin
  Table1.CancelRange;
  SpeedButton9.Down := False;
  Table1.IndexName := AvailableIndexes.Text;
end;

procedure TDBFTest1.DeleteMDX(Sender: TObject);
var
  AliasPath: string;
  TableName: string;
begin
  {Delete the MDX file associated with this DBF (if any)}
  AliasPath := GetAliasPath(Table1.DatabaseName);
  TableName := 'TESTDATA'; {don't use Table1.TableName since it includes .DBF extension}
  CloseDBF(Sender);
  DeleteFile(AliasPath+'\'+TableName+'.MDX');
  DBFnoMDX(AliasPath+'\'+TableName+'.DBF');
  Table1.IndexName := '';
  AvailableIndexes.Items.Clear;
  OpenDBF(Sender);
end;

procedure TDBFTest1.SearchTable1Click(Sender: TObject);
var
  GetDateDlg: TGetDateDlg;
begin
  if (not Table1.Active) or (Table1.IndexName <> 'SLOT') then
    ShowMessage('Table must be opened, and tag must be set to "SLOT".')
  else
  begin
    GetDateDlg := TGetDateDlg.Create(nil);
    try
      if GetDateDlg.ShowModal = mrOk then
      begin
        Table1.SetKey;
        Table1.FieldByName('CRT_DATE').AsString := GetDateDlg.NewCourtDate.Text;
        Table1.FieldByName('CRT_CODE').AsString := GetDateDlg.NewCourtCode.Text;
        if not Table1.GotoKey then
          ShowMessage('Not found!');
      end;
    finally
      GetDateDlg.Free;
    end;
  end;
end;

procedure TDBFTest1.ShowRecord;
{Show a dBase record number}
{NOTE: it assumes there is a label component named RecordNo.}
var
  myrecprop: RECprops;
  recno: integer;
begin
  table1.updatecursorpos;
  recno := 0;
  dbiGetRecord(table1.handle, dbiNOLOCK, nil, @myrecprop);
  recno := myrecprop.iPhyRecNum;
  RecordNo.Caption := inttostr(recno);
end;

procedure TDBFTest1.DBNavigator1Click(Sender: TObject;
  Button: TNavigateBtn);
begin
  ShowRecord;
end;

procedure TDBFTest1.DataSource1DataChange(Sender: TObject; Field: TField);
begin
  ShowRecord;
end;

procedure TDBFTest1.SpeedButton7Click(Sender: TObject);
begin
  {use Table}
  DBNavigator1.DataSource := DataSource1;
  DBGrid1.DataSource := DataSource1;
end;

procedure TDBFTest1.SpeedButton8Click(Sender: TObject);
begin
  {use SQL Query}
  DBNavigator1.DataSource := DataSource2;
  DBGrid1.DataSource := DataSource2;
end;

procedure TDBFTest1.SpeedButton9Click(Sender: TObject);
begin
  if SpeedButton9.Down then begin
    {use the SLOT index sorted on CRT_DATE + CRT_CODE}
    Table1.IndexName := 'SLOT';
    SetComboBox;
    {put a indexed filter on the table}
    with Table1 do begin
      SetRangeStart;
      FieldByName('CRT_DATE').AsString := '04/04/94';
      FieldByName('CRT_CODE').AsString := 'T001';
      SetRangeEnd;
      FieldByName('CRT_DATE').AsString := '06/06/94';
      FieldByName('CRT_CODE').AsString := 'T002';
      ApplyRange;
      First;
    end;
  end
  else {cancel indexed filter}
    Table1.CancelRange;
end;

procedure TDBFTest1.SpeedButton11Click(Sender: TObject);
{filter non-indexed field INFO1, showing only if start with 'K'}
const {static var}
  filterHandle: hDBIFilter = nil;
begin
  if SpeedButton11.Down then
    begin
      OpenDBF(Sender); {make sure DBF is open prior to setting filter}
      Check(DbiAddFilter(Table1.Handle,
                         LongInt(0),
                         Word(1),
                         FALSE,
                         nil,
                         FilterData,
                         filterHandle));
      Check(DbiActivateFilter(Table1.Handle, filterHandle));
      Table1.Refresh;
    end
  else
    begin
      DbiDeactivateFilter(Table1.Handle, nil);
      Table1.Refresh;
    end;
end;

function FilterData(ulClientData : LongInt;
                    pRecBuf      : Pointer;
                    iPhyRecNum   : Longint
                    ): Integer;
var
  Buffer: array[0..255] of char; { holds info on specific field}
  cInfo: string;
  isFieldBlank: bool;
  nINFO1: integer;
begin
  with DBFTest1.Table1 do
    begin
      {get this record's INFO1 (non-indexed field) value}
      nINFO1 := FieldByName('INFO1').Index+1;
      Check(DbiGetField(Handle, nINFO1, pRecBuf, @Buffer, isFieldBlank));
      cInfo := StrPas(Buffer);
      {filter using the following condition(s)}
      if (cInfo[1] = 'K') or (LeftStr(cInfo,4) = 'WEDW') then
        Result := 1  {let it through}
      else
        Result := 0; {don't show it}
    end;
end;

end.

